<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

namespace mod_splernstrategien;

defined('MOODLE_INTERNAL') || die();


/**
 * Library class with mostly static helper functions.
 * 
 * \todo\refactor Move all state machine logic into the statemachine class, this does not belong 
 * here at all.
 * \todo\refactor Use templates instead of echoing pure HTML. 
 * 
 * @package   mod_splernstrategien
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class lib {   

    public static function prepareRender($fsm)
    {
        $user = $fsm->user;
        $strategienObj = $fsm->state;
        
        try {
            $data = new \stdClass();
            $data->asseturl = "https://www.orca.nrw/"; //state=0,27,42,76
            $data->state = $strategienObj->state;
            $data->strang = $strategienObj->strang;
            $data->name = $strategienObj->name;
            $data->description = $strategienObj->description;
            $data->abschnitt = $strategienObj->abschnitt;
            $data->formAction = $_SERVER["PHP_SELF"];
            $data->answers = $user->answers;
            
            //anfang questions-bearbeitung
            $questionsArr = [];
            $questions = $strategienObj->questions;
            if ($questions) {
                // check for loop on same state (can be caused by wrong answers)
                if ($fsm->state->state === $fsm->laststate->state) {
                    $data->detected_loop = true;
                }

                // calculate correct answers
                if ($strategienObj->type == "check") {
                    $arrayCorrectAnswers = self::checkAnswers($user, $strategienObj);
                    foreach ($arrayCorrectAnswers as $k => $v) {
                        $data->$k = $v;
                    }
                }
                if (($strategienObj->type == "abfrage")
                    || ($strategienObj->type == "virtual")  || ($strategienObj->type == "abfrage_video")
                    || ($strategienObj->type == "trigger1") || ($strategienObj->type == "trigger2")
                    || ($strategienObj->type == "trigger3")
                ) {
                    $questionsObj = json_decode($questions);

                    if (!empty($questionsObj)) {
                        foreach ($questionsObj as $htmlElementDescriptionObj) {
                            foreach ($htmlElementDescriptionObj as $val) {
                                if (($val->type == "radio") || ($val->type == "select")) {
                                    $optionsArr = [];
                                    $nameOfElementInDescription = $val->name; // z.b. type=radio, name=geschlecht
                                    //welche options hat radiobox oder select? default? $optionsArr[option => default], z.b. weiblich => 1, maennlich => 0
                                    foreach ($val->options as $option) {
                                        $valueDefault = 0;
                                        if (property_exists($option, 'default')) {
                                            $valueDefault = 1;
                                        }
                                        $optionsArr[$option->name] = $valueDefault;
                                    }

                                    $checked = NULL;
                                    if (!empty($data->answers)) {
                                        //suche checked
                                        foreach ($data->answers as $nameOfElementInAnswers => $optionChecked) {
                                            if ($nameOfElementInAnswers == $nameOfElementInDescription) {
                                                $checked = $optionChecked; //in html, z.B weiblich
                                                foreach ($optionsArr as $key => $value) {
                                                    $questionsArr[$key] = false;
                                                }
                                                $questionsArr[$optionChecked] = true;
                                            }
                                        }
                                    }
                                    if (!$checked) {
                                        //nicht gecheckt: suche default
                                        foreach ($optionsArr as $key => $value) {
                                            $questionsArr[$key] = $value;
                                        }
                                    }
                                    $data->questions = $questionsArr;
                                }
                            } //foreach ($htmlElementDescriptionObj as $val)
                        } //foreach ($questionsObj as $htmlElementDescriptionObj) {
                    } //if (!empty($questionsObj)) {
                } //if ($strategienObj->type == "abfrage") {// bis jetzt noch nicht impl.
            }
            $data->choicename = "choice_" . $data->state;
            $data->wayback = $strategienObj->wayback;

            if ($strategienObj->state == 15) {   //brauchen wir nur in state 15 
                // bearbeitung finished(?) stränge
                $finished = $user->finished_array();
                $disabled_1_2_3 = 0;
                if ($finished) {
                    $data->disabled1 = "";
                    $data->disabled2 = "";
                    $data->disabled3 = "";
                    $data->disabled_1_2_3 = "";
                    if ($finished[0]) {
                        $data->disabled1 = "disabled";
                        $disabled_1_2_3 = 1;
                    }
                    if ($finished[1]) {
                        $data->disabled2 = "disabled";
                        $disabled_1_2_3 += 1;
                    }
                    if ($finished[2]) {
                        $data->disabled3 = "disabled";
                        $disabled_1_2_3 += 1;
                    }
                    if ($disabled_1_2_3 == 3) {
                        $data->disableButton = "disabled";
                    }
                    if ($disabled_1_2_3 == 0) {
                        $data->first_time = "true";
                    }
                }
            }

            return $data;

        } catch (\Exception $e) {
            echo "Exeption " . $e->getMessage();
        }

    } //end prepareRender


    public static function renderData($fsm)
    {
        global $PAGE, $OUTPUT;
        
        $user = $fsm->user;
        $strategienObj = $fsm->state;
        
        if ($strategienObj && $strategienObj->template) {
            $data = self::prepareRender($fsm);
            $data->strDelay = self::getStrDelay($user->delay_left());
            $templatename = __NAMESPACE__ . '/' . $strategienObj->template;

            /****************
             * Requirements *
             * **************/
            $PAGE->requires->css('/mod/splernstrategien/css/lernstrategien.css');
            
            // \todo\refactor These are needed because of some templates which use list_sort or 
            // dragdrop JS libs. These should be loaded from the template somehow...
            $PAGE->requires->css('/mod/splernstrategien/css/dnd.css');
            $PAGE->requires->css('/mod/splernstrategien/css/265.css');

            // Start Output of Page
            echo $OUTPUT->header();
            if (debugging()){
                echo "<pre>";
                print_r($strategienObj);
                echo "</pre>";
            }
            echo "<div  id=\"lernstrategien\" class=\"lernstrategien\">\n";

            if (($strategienObj->type != "virtual") || ($strategienObj->type != "checkingTraces")) {
                echo $OUTPUT->render_from_template($templatename, $data);
            }
            
            echo "</div>\n";

            if ($strategienObj->end_strang) {
                $user->set_finished($strategienObj->strang);
            }

            if (debugging()) {
                $debug_answers = "<div class='wrapperAntworten'>" .
                    (!empty($user->answers)) ? json_encode($user->answers) : "{}" . "</div>";

                $debug_stateinfo = "<br>State: " . $strategienObj->state;
                $debug_stateinfo .= "<br>Strang: " . $strategienObj->strang;
                $debug_stateinfo .= "<br>Name: " . $strategienObj->name;
                $debug_stateinfo .= "<br>Description: " . $strategienObj->description;

                $w = $strategienObj->waysout;

                $debug_stateinfo .= " <br> vorwärts: ";
                if ($w && strpos($w, ",")) {
                    $arr = str_getcsv($w, ",");
                    $a = explode(",", $w);
                    if (!empty($a)) {
                        foreach ($a as $key => $value) {
                            $debug_stateinfo .= self::gethref($user, trim($value), $key, count($arr));
                            $debug_stateinfo .= " ";
                        }
                    }
                }
                else {
                    $debug_stateinfo .= self::gethref($user, trim($w), -5, 1);
                }

                $debug_stateinfo .= " <br> zurück: ";
                $b = $strategienObj->wayback;
                $debug_stateinfo .= self::gethref($user, trim($b), -5, 1);
                echo "<pre>";
                print_r($data);
                echo "</pre>";
            }
            debugging($debug_answers, DEBUG_DEVELOPER);
            debugging($debug_stateinfo, DEBUG_DEVELOPER);
            if (debugging()) {
                phpinfo(INFO_VARIABLES);
            }
            echo $OUTPUT->footer();
        }
    } //renderData

    public static function checkAnswers($user, $state)
    {
        //check correct Answers from questions.check with Answers
        $correctAnswArr = [];
        $correctAnswers = 0;
        $arrayCorrectAnswers = [];
        $questionsObj = json_decode($state->questions);
        if (!empty($questionsObj)) {
            foreach ($questionsObj as $i => $value) {
                if (($i == "check") && !empty($value)) {
                    $correctAnswObj = $value;
                    foreach ($correctAnswObj as $k => $v) {
                        if (array_key_exists($k, $user->answers) && ($user->answers[$k] == $v)) {
                            $arrayCorrectAnswers[$k . 'enabled'] = true;
                            $correctAnswers++;
                        } else {
                            $arrayCorrectAnswers[$k . 'enabled'] = false;
                        }
                    }
                    $arrayCorrectAnswers['correctAnswers'] = $correctAnswers;
                    return $arrayCorrectAnswers;
                }
            }
        }
    }

    public static function getStrDelay($intervalSeconds)
    {
        $str = "";
        $secondsInAMinute = 60;
        $secondsInAnHour  = 60 * $secondsInAMinute;
        $secondsInADay    = 24 * $secondsInAnHour;
        // extract days
        $days = floor($intervalSeconds / $secondsInADay);

        // extract hours
        $hourSeconds = $intervalSeconds % $secondsInADay;
        $hours = floor($hourSeconds / $secondsInAnHour);

        // extract minutes
        $minuteSeconds = $hourSeconds % $secondsInAnHour;
        $minutes = floor($minuteSeconds / $secondsInAMinute);

        // extract the remaining seconds
        $remainingSeconds = $minuteSeconds % $secondsInAMinute;
        $seconds = ceil($remainingSeconds);

        if ($days) {
            if ($days == 1) $str =  $days . " Tag";
            else $str =  $days . " Tagen";
        }
        if ($hours) {
            if ($str) $str = $str . " ";
            if ($hours == 1) $str = $str . $hours . " Stunde";
            else $str = $str . $hours . " Stunden";
        }
        if ($minutes) {
            if ($str) $str = $str . " ";
            if ($minutes == 1) $str = $str . $minutes . " Minute";
            else $str = $str . $minutes . " Minuten";
        }
        if ($seconds) {
            if ($str) $str = $str . " ";
            if ($seconds == 1)  $str = $str . $seconds . " Sekunde";
            else $str = $str . $seconds . " Sekunden";
        }
        return $str;
    }

    public static function gethref($user, $state, $flag, $count)
    {
        // debug only !!
        //state - ein link, s.vorwards unten
        //vorwärts: 16 32 48 64 - array von links, index in array $trigger[$flag], erste 3 - wie trigger zeigt, 4.te - einfach link
        //$flag - 0,1,2 (strang nummer minus 1)
        //$flag = -5 - wird einfach link
        //$count - length of way's array, if $count < 3 trigger do not work

        // global $COURSE;
        $return = "";
        $finished = [];
        $link = new \moodle_url('/mod/splernstrategien/view.php', array('state' => $state));
        $contenta = 'href=' . '"' . $link . '"';
        if (($flag > -1) && ($flag < 3) && ($count > 2)) {
            $finished[0] = $user->epa_finished;
            $finished[1] = $user->epb_finished;
            $finished[2] = $user->wp_finished;

            if ($finished[$flag]) {
                $contenta = " href='javascript:void(0)' style='cursor: default; pointer-events: none; text-decoration: none; opacity: .4;' ";
            }
        }
        $return .= '<a ' . $contenta . ' >';
        $return .= $state;
        $return .= '</a>';
        return $return;
    }
    
}
