<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

/**
 * Task for sending notifications.
 * 
 * We send notification on end of delay if user has agreed to receive 
 * them by answering yes in state 121.
 * 
 * @package   mod_splernstrategien
 * @category  task
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace mod_splernstrategien\task;

defined('MOODLE_INTERNAL') || die();

/**
 * A scheduled task, starts trigger
 */
class trigger_mail extends \core\task\scheduled_task
{
    /**
     * Return the task's name as shown in admin screens.
     *
     * @return string
     */

    public function get_name()
    {
        // Shown on admin screens
        return get_string('sendtriggermails', 'splernstrategien');
    }

    /**
     * Execute the task.
     */
    public function execute()
    {
        global $CFG;
        //require_once($CFG->dirroot . '/config.php');
        global $DB;
        mtrace("Task triggermail in Lernstrategien started");
        $table1 = "splernstrategien_users";
        $subject = "Lernstrategien – Freischaltung der nächsten Triggereinheit";
        $messagetext = <<<EOT1
Liebe Nutzende, 

Sie haben vor kurzem das Grundmodul des Kurses „Lernstrategien“ 
abgeschlossen und dabei Möglichkeiten kennengelernt, Motivationsprobleme 
zu erkennen und passende Strategien zur Selbstregulation Ihrer 
Motivation anzuwenden. Zum Abschluss des Grundmoduls haben Sie zugestimmt, 
dass wir Sie im Abstand von jeweils einer Woche über die Freischaltung 
der drei Triggereinheiten zur Nachbereitung und Vertiefung des Erlernten
informieren. 

Wir möchten Ihnen mitteilen, dass die nächste Triggereinheit für Sie 
freigeschaltet ist. Öffnen Sie, um diese zu erreichen, wieder den Kurs 
und melden Sie sich mit Ihrem bisher benutzen Profil an. Sie haben den 
Kurs auf einer Abschlussseite mit weiterführenden Angeboten verlassen. 
Auf diese werden Sie nach dem Login wieder geleitet. Am unteren rechten 
Rand dieser Seite können Sie von nun an auf „Weiter“ klicken, um zur 
nächsten Triggereinheit fortzuschreiten.

Ihr Link zum Kurs: {{link}}

Wir wünschen Ihnen weiterhin viel Erfolg bei der Anwendung
der erlernten Strategien! 

Mit freundlichen Grüßen
Ihr Kursentwicklungsteam


Bitte beachten Sie, dass es sich hierbei um eine automatisiert 
versandte und nicht beantwortbare Nachricht handelt.
EOT1;
        $messagehtml = "<p>Liebe Nutzende,</br> </br> 
        Sie haben vor kurzem das Grundmodul des Kurses „Lernstrategien“ 
        abgeschlossen und dabei Möglichkeiten kennengelernt, Motivationsprobleme 
        zu erkennen und passende Strategien zur Selbstregulation Ihrer 
        Motivation anzuwenden. Zum Abschluss des Grundmoduls haben Sie zugestimmt, 
        dass wir Sie im Abstand von jeweils einer Woche über die Freischaltung 
        der drei Triggereinheiten zur Nachbereitung und Vertiefung des Erlernten
        informieren. <br /> <br />
        
        Wir möchten Ihnen mitteilen, dass 
        <a href={{link}} title=\"zur nächsten Triggereinheit\">die nächste Triggereinheit</a>
        für Sie freigeschaltet ist. Öffnen Sie, um diese zu erreichen, wieder den Kurs 
        und melden Sie sich mit Ihrem bisher benutzen Profil an. Sie haben den 
        Kurs auf einer Abschlussseite mit weiterführenden Angeboten verlassen. 
        Auf diese werden Sie nach dem Login wieder geleitet. Am unteren rechten 
        Rand dieser Seite können Sie von nun an auf „Weiter“ klicken, um 
        <a href={{link}} title=\"zur nächsten Triggereinheit\">zur 
        nächsten Triggereinheit</a> fortzuschreiten.<br /><br />
        
        Wir wünschen Ihnen weiterhin viel Erfolg bei der Anwendung 
        der erlernten Strategien! <br /><br />
        
        Mit freundlichen Grüßen<br />
        Ihr Kursentwicklungsteam
        <br /><br />
        Bitte beachten Sie, dass es sich hierbei um eine automatisiert 
        versandte und nicht beantwortbare Nachricht handelt.</p>";

        if (!$CFG->messaging) {
            mtrace("Bitte prüfen Sie Aktivierung der Mitteilungen (Festgelegt in 'Website-Administration / Mitteilungen / Mitteilungseinstellungen / Mitteilungen aktivieren' ");
            mtrace("Jetzt Mitteilungen aktivieren = " . $CFG->messaging);
        } else {
            $list = $DB->get_records_select($table1, "delay > 0");

            foreach ($list as $key => $value) {
                $userto =  $DB->get_record(
                    'user',
                    ['id' => $value->user_id],
                    $fields =    '*',
                    $strictness = IGNORE_MISSING
                );
                // get user-obj and read allowance for triggermails
                list($course, $cm) = get_course_and_cm_from_cmid($value->moodle_id, 'splernstrategien');
                $ls_user = \mod_splernstrategien\user::from_moodleuser($userto, $course->id, $cm->id);
                $triggermail_allowed = $ls_user->answers['choice_121']; // allowance is given in state 121
                $link = (new \moodle_url('/mod/splernstrategien/view.php?id=' . $value->moodle_id))->out(false);
                if ($ls_user->delay_left() < 0 && $triggermail_allowed == "yes") {
                    $message = new \core\message\message();
                    $message->component         = 'mod_splernstrategien';
                    $message->name              = 'triggermail'; //this is the message name from messages.php
                    $message->courseid          = $value->course;
                    $message->contexturl        = $link;
                    $message->contexturlname    = 'mit den Lernstrategien weitermachen';
                    $message->userfrom          = \core_user::get_noreply_user();
                    $message->userto            = $userto;
                    $message->subject           = $subject;
                    $message->fullmessage       = str_replace("{{link}}", $link, $messagetext);
                    $message->fullmessageformat = FORMAT_PLAIN;
                    $message->fullmessagehtml   = str_replace("{{link}}", $link, $messagehtml);
                    $message->notification      = 1; //this is only set to 0 for personal messages between users
                    $result = message_send($message);
                    if ($result && $result > 0) {
                        $ls_user->delay = 0;
                        $ls_user->save();
                        mtrace("Email({$userto->email}) für Benutzer id = {$userto->id}");
                    }
                }
            }
            mtrace("Task triggermail Lernstrategien finished");
        }
    }
}
