<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

// This file is used when adding/editing a module to a course.
// It contains the elements that will be displayed on the form responsible
// for creating/installing an instance of your module.
// The class in the file should be called mod_<modname>_mod_form
//

/**
 * The main splernstrategien configuration form.
 *
 * @package   mod_splernstrategien
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot . '/course/moodleform_mod.php');

/**
 * Module instance settings form.
 *
 */
class mod_splernstrategien_mod_form extends moodleform_mod
{

    /**
     * Defines forms elements
     */
    public function definition()
    {
        global $CFG;

        $mform = $this->_form;

        // Adding the "general" fieldset, where all the common settings are shown.
        $mform->addElement('header', 'general', get_string('general', 'form'));

        // Adding the standard "name" field.
        $mform->addElement('text', 'name', get_string('splernstrategienname', 'splernstrategien'), array('size' => '64'));

        if (!empty($CFG->formatstringstriptags)) {
            $mform->setType('name', PARAM_TEXT);
        } else {
            $mform->setType('name', PARAM_CLEANHTML);
        }

        $mform->addRule('name', null, 'required', null, 'client');
        $mform->addRule('name', get_string('maximumchars', '', 255), 'maxlength', 255, 'client');
        $mform->addHelpButton('name', 'splernstrategienname', 'splernstrategien');

        // Adding the standard "intro" and "introformat" fields.
        if ($CFG->branch >= 29) {
            $this->standard_intro_elements();
        } else {
            $this->add_intro_editor();
        }

        // Adding the rest of splernstrategien settings, spreading all them into this fieldset
        // ... or adding more fieldsets ('header' elements) if needed for better logic.

        //$mform->addElement('static', 'label1', 'splernstrategiensettings', get_string('splernstrategiensettings', 'splernstrategien'));
        //$mform->addElement('header', 'splernstrategienfieldset', get_string('splernstrategienfieldset', 'splernstrategien'));

        // Add standard elements.
        $this->standard_coursemodule_elements();

        // Add standard buttons.
        $this->add_action_buttons();
    }

    /**
     * Add elements for setting the custom completion rules.
     *
     * @category completion
     * @return array List of added element names, or names of wrapping group elements.
     */
    public function add_completion_rules()
    {
        $mform = $this->_form;
        $group = [
            $mform->createElement(
                'static',
                'completiondescription', // as of 4.3 we must use: $suffix = $this->get_suffix(); for element-id
                '',
                get_string('completionrules:description', 'splernstrategien')
            ),
        ];
        $mform->addGroup(
            $group,
            'completiongroup', // as of 4.3 we must use: $suffix = $this->get_suffix(); for element-id
            '',
            [' '],
            false
        );
        return ['completiongroup'];
    }

    /**
     * Called during validation to see whether some activity-specific completion rules are selected.
     *
     * @param array $data Input data not yet validated.
     * @return bool True if one or more rules is enabled, false if none are.
     */
    public function completion_rule_enabled($data)
    {
        return True; //there is no choice, all rules are alway enabled
    }
}
